"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const core_1 = require("@oclif/core");
const core_2 = require("@salesforce/core");
const kit_1 = require("@salesforce/kit");
const orgs_1 = require("../../utils/orgs");
const componentSetBuilder_1 = require("../../utils/componentSetBuilder");
const tableBuilder_1 = require("../../utils/tableBuilder");
const testLevel_1 = require("../../utils/testLevel");
const progressBar_1 = require("../../utils/progressBar");
const config_1 = require("../../utils/config");
core_2.Messages.importMessagesDirectory(__dirname);
const messages = core_2.Messages.loadMessages('@salesforce/plugin-deploy-retrieve-metadata', 'deploy.metadata');
class DeployMetadata extends core_1.Command {
    async run() {
        const flags = (await this.parse(DeployMetadata)).flags;
        const componentSet = await componentSetBuilder_1.ComponentSetBuilder.build({
            directory: flags['deploy-dir'],
            manifest: (flags.manifest && {
                manifestPath: flags.manifest,
                directoryPaths: await orgs_1.getPackageDirs(),
            }),
            metadata: flags.metadata && {
                metadataEntries: flags.metadata,
                directoryPaths: await orgs_1.getPackageDirs(),
            },
        });
        const targetOrg = await orgs_1.resolveTargetOrg(flags['target-org']);
        this.log(`${os_1.EOL}${messages.getMessage('deploy.metadata.api', [targetOrg, config_1.resolveRestDeploy()])}${os_1.EOL}`);
        const deploy = await componentSet.deploy({
            usernameOrConnection: targetOrg,
            apiOptions: {
                testLevel: flags['test-level'],
            },
        });
        if (!this.jsonEnabled()) {
            new progressBar_1.DeployProgress(deploy).start();
        }
        const result = await deploy.pollStatus(500, kit_1.Duration.minutes(flags.wait).seconds);
        const fileResponses = tableBuilder_1.asRelativePaths((result === null || result === void 0 ? void 0 : result.getFileResponses()) || []);
        if (!this.jsonEnabled()) {
            tableBuilder_1.displayHumanReadableResults(fileResponses);
        }
        return fileResponses;
    }
    toSuccessJson(result) {
        return { status: 0, result };
    }
    toErrorJson(err) {
        return { status: process.exitCode || 1, err };
    }
}
exports.default = DeployMetadata;
DeployMetadata.description = messages.getMessage('description');
DeployMetadata.summary = messages.getMessage('summary');
DeployMetadata.examples = messages.getMessages('examples');
DeployMetadata.flags = {
    metadata: core_1.Flags.string({
        char: 'm',
        summary: messages.getMessage('flags.metadata.summary'),
        multiple: true,
    }),
    manifest: core_1.Flags.string({
        char: 'x',
        description: messages.getMessage('flags.manifest.description'),
        summary: messages.getMessage('flags.manifest.summary'),
    }),
    'deploy-dir': core_1.Flags.string({
        char: 'd',
        description: messages.getMessage('flags.deploy-dir.description'),
        summary: messages.getMessage('flags.deploy-dir.summary'),
        multiple: true,
    }),
    'target-org': core_1.Flags.string({
        char: 't',
        description: messages.getMessage('flags.target-org.description'),
        summary: messages.getMessage('flags.target-org.summary'),
    }),
    'test-level': core_1.Flags.string({
        char: 'l',
        description: messages.getMessage('flags.test-level.description'),
        summary: messages.getMessage('flags.test-level.summary'),
        options: Object.values(testLevel_1.TestLevel),
        default: testLevel_1.TestLevel.NoTestRun,
    }),
    wait: core_1.Flags.integer({
        char: 'w',
        summary: messages.getMessage('flags.wait.summary'),
        description: messages.getMessage('flags.wait.description'),
        default: 33,
    }),
};
//# sourceMappingURL=metadata.js.map